Don't complain about a socket EOF unless it affects a read.
[rsync/rsync.git] / cleanup.c
index 2ded302..ae185d4 100644 (file)
--- a/cleanup.c
+++ b/cleanup.c
@@ -4,7 +4,7 @@
  * Copyright (C) 1996-2000 Andrew Tridgell
  * Copyright (C) 1996 Paul Mackerras
  * Copyright (C) 2002 Martin Pool
- * Copyright (C) 2003-2007 Wayne Davison
+ * Copyright (C) 2003-2009 Wayne Davison
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
 
 extern int am_server;
 extern int am_daemon;
+extern int am_sender;
+extern int am_generator;
 extern int io_error;
 extern int keep_partial;
-extern int log_got_error;
+extern int got_xfer_error;
+extern int protocol_version;
+extern int output_needs_newline;
 extern char *partial_dir;
 extern char *logfile_name;
 
@@ -94,17 +98,21 @@ pid_t cleanup_child_pid = -1;
 NORETURN void _exit_cleanup(int code, const char *file, int line)
 {
        static int cleanup_step = 0;
-       static int exit_code = 0;
+       static int exit_code = 0, exit_line = 0;
+       static const char *exit_file = NULL;
        static int unmodified_code = 0;
 
        SIGACTION(SIGUSR1, SIG_IGN);
        SIGACTION(SIGUSR2, SIG_IGN);
 
-       if (exit_code) /* Preserve first error code when recursing. */
+       if (exit_code) { /* Preserve first exit info when recursing. */
                code = exit_code;
+               file = exit_file;
+               line = exit_line;
+       }
 
        /* If this is the exit at the end of the run, the server side
-        * should not attempt to output a message (see log.c). */
+        * should not attempt to output a message (see log_exit()). */
        if (am_server && code == 0)
                am_server = 2;
 
@@ -114,8 +122,15 @@ NORETURN void _exit_cleanup(int code, const char *file, int line)
 #include "case_N.h" /* case 0: cleanup_step++; */
 
                exit_code = unmodified_code = code;
+               exit_file = file;
+               exit_line = line;
+
+               if (output_needs_newline) {
+                       fputc('\n', stdout);
+                       output_needs_newline = 0;
+               }
 
-               if (verbose > 3) {
+               if (DEBUG_GTE(EXIT, 2)) {
                        rprintf(FINFO,
                                "_exit_cleanup(code=%d, file=%s, line=%d): entered\n",
                                code, file, line);
@@ -154,7 +169,8 @@ NORETURN void _exit_cleanup(int code, const char *file, int line)
                /* FALLTHROUGH */
 #include "case_N.h"
 
-               io_flush(FULL_FLUSH);
+               if (!code || am_server || (!am_sender && !am_generator))
+                       io_flush(FULL_FLUSH);
 
                /* FALLTHROUGH */
 #include "case_N.h"
@@ -174,23 +190,36 @@ NORETURN void _exit_cleanup(int code, const char *file, int line)
                                code = exit_code = RERR_DEL_LIMIT;
                        if (io_error & IOERR_VANISHED)
                                code = exit_code = RERR_VANISHED;
-                       if (io_error & IOERR_GENERAL || log_got_error)
+                       if (io_error & IOERR_GENERAL || got_xfer_error)
                                code = exit_code = RERR_PARTIAL;
                }
 
-               if (code || am_daemon || (logfile_name && (am_server || !verbose)))
+               /* If line < 0, this exit is after a MSG_ERROR_EXIT event, so
+                * we don't want to output a duplicate error. */
+               if ((code && line > 0)
+                || am_daemon || (logfile_name && (am_server || !INFO_GTE(STATS, 1))))
                        log_exit(code, file, line);
 
                /* FALLTHROUGH */
 #include "case_N.h"
 
-               if (verbose > 2) {
+               if (DEBUG_GTE(EXIT, 1)) {
                        rprintf(FINFO,
                                "_exit_cleanup(code=%d, file=%s, line=%d): "
                                "about to call exit(%d)\n",
                                unmodified_code, file, line, code);
                }
 
+               /* FALLTHROUGH */
+#include "case_N.h"
+
+               if (exit_code && exit_code != RERR_STREAMIO && exit_code != RERR_SIGNAL1
+                && (protocol_version >= 31 || (!am_sender && !am_generator))) {
+                       if (line > 0)
+                               send_msg_int(MSG_ERROR_EXIT, exit_code);
+                       noop_io_until_death();
+               }
+
                /* FALLTHROUGH */
 #include "case_N.h"